import { jsx as _jsx } from "react/jsx-runtime";
import { ClientSocketManager } from "@tapsioss/client-socket-manager";
import * as React from "react";
import { ConnectionStatus } from "./constants.js";
import { SocketContext } from "./Context.js";
const __SINGLETON_REFS__ = {};
const SocketClientProvider = (props) => {
    var _a;
    const { children, uri, ...options } = props;
    const [clientInstance, setClientInstance] = React.useState((_a = __SINGLETON_REFS__[uri]) !== null && _a !== void 0 ? _a : null);
    const [connectionStatus, setConnectionStatus] = React.useState(ConnectionStatus.DISCONNECTED);
    React.useEffect(() => {
        var _a;
        if (!__SINGLETON_REFS__[uri]) {
            const client = new ClientSocketManager(uri, {
                ...options,
                eventHandlers: {
                    ...((_a = options.eventHandlers) !== null && _a !== void 0 ? _a : {}),
                    onSocketConnection() {
                        var _a, _b;
                        (_b = (_a = options.eventHandlers) === null || _a === void 0 ? void 0 : _a.onSocketConnection) === null || _b === void 0 ? void 0 : _b.call(client);
                        setConnectionStatus(ConnectionStatus.CONNECTED);
                    },
                    onSocketDisconnection(reason, details) {
                        var _a, _b;
                        (_b = (_a = options.eventHandlers) === null || _a === void 0 ? void 0 : _a.onSocketDisconnection) === null || _b === void 0 ? void 0 : _b.call(client, reason, details);
                        setConnectionStatus(ConnectionStatus.DISCONNECTED);
                    },
                    onReconnecting(attempt) {
                        var _a, _b;
                        (_b = (_a = options.eventHandlers) === null || _a === void 0 ? void 0 : _a.onReconnecting) === null || _b === void 0 ? void 0 : _b.call(client, attempt);
                        setConnectionStatus(ConnectionStatus.RECONNECTING);
                    },
                },
            });
            setClientInstance(client);
            __SINGLETON_REFS__[uri] = client;
        }
        else {
            throw new Error([
                `ClientSocketManager: An active client already exists for the URI "${uri}".`,
                "Multiple clients for the same URI are not allowed.",
            ].join(" "));
        }
        return () => {
            if (__SINGLETON_REFS__[uri]) {
                __SINGLETON_REFS__[uri].dispose();
                __SINGLETON_REFS__[uri] = null;
            }
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const ctx = React.useMemo(() => ({
        connectionStatus,
        get socket() {
            if (!clientInstance)
                return null;
            if (clientInstance.disposed)
                return null;
            return clientInstance;
        },
    }), [connectionStatus, clientInstance]);
    return (_jsx(SocketContext.Provider, { value: ctx, children: children }));
};
export default SocketClientProvider;
